/*
 * This file is part of din.
 *
 * din is copyright (c) 2006 - 2011 S Jagannathan <jag@dinisnoise.org>
 * For more information, please visit http://dinisnoise.org
 *
 * din is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * din is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with din.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
#ifndef __viewwin
#define __viewwin

#include <box.h>
#include <string>
using namespace std;

struct badviewport {};

struct viewport { // opengl

	int cx, cy; // center
	int xmax, ymax;
	float xmax_1, ymax_1; // 1.0 / xmax , 1.0 / ymax
  int width, height;

	// for handle of vertex/tangent in curve editor
	static int HANDLE_RADIUS;
	static float HANDLE_FACTOR;

	viewport ();
	void setup_surface (int width, int height, int depth);
	bool inside (int x, int y);
	void calc (int w, int h);

};

struct window : box<float> { // curve editor


	static int PAN_RATE, ZOOM_RATE; // num pan, zooms per second
	static float PAN_AMOUNT, ZOOM_AMOUNT; // pan, zoom amount per key press
	static double PAN_REPEAT, ZOOM_REPEAT; // key repeat time


	static void calc_repeats ();

	float pan_x, pan_y;
	void calc_panxy ();

	float width_1, height_1; // 1.0 / width, 1.0 / height
	float aspect_ratio;

	window ();
	window (float l, float b, float r, float t);
	void set (float l, float b, float r, float t);
	void get (float& l, float& b, float& r, float& t);
	void panx (int dir);
	void pany (int dir);
	void zoom (int dir);
	void project ();

	float handle_radius, handle_radius2;
	void calc_handle_radius ();


};

inline void view2win (int vx, int vy, float& wx, float& wy, viewport& view, window& win) {

	float xr = vx * view.xmax_1;
	float yr = 1 - vy * view.ymax_1;

	wx = win.left + xr * win.width;
	wy = win.bottom + yr * win.height;

}

inline void win2view (float wx, float wy, int& vx, int& vy, window& win, viewport& view) {
	float xr = (wx - win.left) * win.width_1;
	float yr = (wy - win.bottom) * win.height_1;
	vx = (int) (xr * view.xmax);
	vy = (int) (yr * view.ymax);

}

#endif
