/*
 * This file is part of din.
 *
 * din is copyright (c) 2006 - 2011 S Jagannathan <jag@dinisnoise.org>
 * For more information, please visit http://dinisnoise.org
 *
 * din is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * din is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with din.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
#include "tokenizer.h"
#include "command.h"
#include "tcl_interp.h"
#include "random.h"
#include <iostream>
#include <stdlib.h>

using namespace std;

extern cmdlist cmdlst;

int tcl_run (ClientData cd, Tcl_Interp* ti, int objc, Tcl_Obj* CONST objv[]) {

	string cmd (Tcl_GetString (objv[0]));
	command* cmdp = cmdlst.cmds[(intptr_t)cd];
	if (cmdp) {
		cmdlst.result = "";
		vector<string> args; for (int i = 1; i < objc; ++i) args.push_back (Tcl_GetString(objv[i]));
		tokenizer tz (args);cmdp->operator() (tz);
		Tcl_SetResult (ti, (char *) cmdlst.result.c_str(), TCL_VOLATILE);
	}
	return 0;
}

void add_commands (Tcl_Interp* ti);

tcl_interp::tcl_interp () {

	Tcl_FindExecutable (0);

	interp = Tcl_CreateInterp ();
	if (!interp) {
		cout << "!!! Could not create Tcl interpreter !!!" << endl;
		exit (1);
	}

	extern double MIDI_BPM, TAP_BPM, TIME_NOW;
	add_commands (interp);
	Tcl_LinkVar (interp, "midibpm", (char *) &MIDI_BPM, TCL_LINK_DOUBLE);
	Tcl_LinkVar (interp, "tapbpm", (char *) &TAP_BPM, TCL_LINK_DOUBLE);
	Tcl_LinkVar (interp, "timenow", (char *) &TIME_NOW, TCL_LINK_DOUBLE);

	cout << "+++ created Tcl interpreter +++" << endl;

}

const char* tcl_interp::operator () (const string& cmd) {
	Tcl_Obj* script = Tcl_NewStringObj (cmd.c_str(), -1);
	Tcl_IncrRefCount (script);
	Tcl_EvalObjEx (interp, script, 0);
	Tcl_DecrRefCount (script);
	result = Tcl_GetStringResult (interp);
	return result;
}

tcl_interp::~tcl_interp () {
	if (interp) {
		Tcl_UnlinkVar (interp, "midibpm");
		Tcl_UnlinkVar (interp, "tapbpm");
		Tcl_UnlinkVar (interp, "timenow");
		Tcl_DeleteInterp (interp);
	}
}
