/*
 * This file is part of din.
 *
 * din is copyright (c) 2006 - 2011 S Jagannathan <jag@dinisnoise.org>
 * For more information, please visit http://dinisnoise.org
 *
 * din is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * din is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with din.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
#include "ui_list.h"
#include "din.h"
#include "morse_code.h"
#include "console.h"
#include "curve_editor.h"
#include "tcl_interp.h"
#include "command.h"
#include "chrono.h"
#include "font_editor.h"

extern console cons;
extern cmdlist cmdlst;
extern din din0;
extern morse_code mc;
extern curve_editor delayed;
extern string dotdin;
extern tcl_interp* p_interp;
extern font fnt;

const int ui_list::key [] = {SDLK_1, SDLK_2, SDLK_3, SDLK_4, SDLK_5, SDLK_6, SDLK_7, SDLK_8};
ui* ui_list::ed [] = {0};

ui_list::ui_list () :  fed (fnt, "font.ed") {

	current = 0;

	crved = 0;

	uis.push_back (&din0);
	uis.push_back (&din0.stred);
	uis.push_back (&din0.waved);
	uis.push_back (&din0.chaned);
	uis.push_back (&din0.moded);
	uis.push_back (&din0.gated);
	uis.push_back (&delayed);
	uis.push_back (&din0.octed);
	uis.push_back (&din0.droneed);
	uis.push_back (&din0.comed);
	uis.push_back (&mc.ed);
	uis.push_back (&fed);

	esct = -1;

}

void ui_list::set_current (ui* u, int ised) {
	if (ised) crved = dynamic_cast<curve_editor*>(u); else crved = 0;
	if (current) current->leave ();
		current = u;
	current->enter ();
	cons << console::yellow << "@ " << current->name << eol;
}

bool ui_list::handle_input () {

	current->calc_win_mouse ();

	for (int i = 0, j = uis.size(); i < j; ++i) uis[i]->bg ();

	if (cons.command_mode == 0) {

		current->handle_input ();

		static int all = 0;
		int num_resonators = din0.resonators.size ();
		if (keypressed (SDLK_F2)) {
			if (num_resonators) {
				if (--din0.cur_resonator < 0) din0.cur_resonator = num_resonators - 1;
				din0.stred.enable (din0.cur_resonator);
				din0.stred.pick (din0.cur_resonator);
				cons << console::green << "active resonator: " << din0.resonators[din0.cur_resonator].strength.name << eol;
				all = 1;
			}
		} else
		if (keypressed (SDLK_F3)) {
			if (num_resonators) {
				if (++din0.cur_resonator >= num_resonators) din0.cur_resonator = 0;
				din0.stred.enable (din0.cur_resonator);
				din0.stred.pick (din0.cur_resonator);
				all = 1;
				cons << console::green << "active resonator: " << din0.resonators[din0.cur_resonator].strength.name << eol;
			}
		} else
		if (keypressed (SDLK_F4)) {
			if (all) {
				din0.stred.enable_all ();
				all = 0;
				cons << console::green << "all resonators enabled." << eol;
			} else {
				if (din0.cur_resonator > -1 && din0.cur_resonator < num_resonators) {
					din0.stred.enable (din0.cur_resonator);
					din0.stred.pick (din0.cur_resonator);
					all = 1;
					cons << console::yellow << "active resonator " << din0.resonators[din0.cur_resonator].strength.name << eol;
				}
			}
		} else if (keypressed (SDLK_RETURN)) { // calc tap bpm

			extern nano_chrono ui_clk;
			extern double TAP_BPM;
			static double last_tapt = ui_clk.secs;
			double dt = ui_clk.secs - last_tapt;
			if (dt) TAP_BPM = 60.0 / dt;
			extern tcl_interp interpreter;
			Tcl_UpdateLinkedVar (interpreter.interp, "tapbpm");
			if (keydown (SDLK_LALT)) cons ("set-now $taptarget 0");
			last_tapt = ui_clk.secs;
		} else for (int i = 0; i < MAX_EDITORS; ++i) {
			if (keypressed (key[i])) {
				if (ed[i]) set_current (ed[i]);
				break;
			}
		}

	}

	cons.handle_input ();

	extern nano_chrono ui_clk;
	if (keypressed (SDLK_ESCAPE)) {
		if (cons.command_mode || (current != &din0)) {
			set_current (&din0, 0);
			cons.command_mode = 0;
			esct = -1;
		} else {
			esc:
			if (esct == -1) {
				cons << console::red << "Press ESC again to quit" << eol;
				esct = ui_clk.secs;
			} else {
				static const float ESC_TIMEOUT = 1; // in seconds
				float dt = ui_clk.secs - esct;
				if (dt > ESC_TIMEOUT) {
					esct = -1;
					goto esc;
				} else {
					extern audio_out aout; aout.stop ();
					extern int quit_now; quit_now = 1;
				}
			}
		}
	}

	return true;
}

bool ui_list::set_editor (const string& name, int screen) {

	if (screen > 0) {

		ui* ei = 0;
		for (int i = 0, j = uis.size(); i < j; ++i) {
			if (uis[i]->name == name) {
				ei = uis[i];
				break;
			}
		}
		if (ei) {
			ed [screen - 1] = ei;
			return true;
		} else {
			cmdlst.result = "bad editor name";
			return false;
		}
	} else {
		cmdlst.result = "bad screen number (valid: 1 to 8)";
		return true;
	}
}

void ui_list::draw () {
	glClear (GL_COLOR_BUFFER_BIT);
		current->draw ();
		cons.draw ();
	SDL_GL_SwapBuffers ();
}
