/*
 * This file is part of din.
 *
 * din is copyright (c) 2006 - 2011 S Jagannathan <jag@dinisnoise.org>
 * For more information, please visit http://dinisnoise.org
 *
 * din is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * din is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with din.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>
#include <console.h>
#include <viewwin.h>
#include <iostream>
using namespace std;

int viewport::HANDLE_RADIUS = 20;
float viewport::HANDLE_FACTOR = 0;

void viewport::setup_surface (int w, int h, int depth) {
  SDL_Surface* s = SDL_SetVideoMode (w, h, depth, SDL_OPENGL | SDL_RESIZABLE);
  if (s) {
		width = s->w;
		height = s->h;
		calc (width, height);
    glViewport (0, 0, width, height);
    cout << "viewport initialised : " << width << ' ' << height << ' ' << depth << endl;
  } else {
    cout << "could not initialise video mode " << width << ' ' << height << ' ' << depth << endl;
		exit (1);
  }
}

void viewport::calc (int w, int h) {
	width = w;
	height = h;
	xmax = width - 1;
	ymax = height - 1;
	xmax_1 = 1.0 / xmax;
	ymax_1 = 1.0 / ymax;
	cx = xmax >> 1;
	cy = ymax >> 1;
	HANDLE_FACTOR = HANDLE_RADIUS * 1. / width;
}

viewport::viewport () {

  if (SDL_Init (SDL_INIT_VIDEO) < 0) {
		cout << "SDL init video failed: " << width << ' ' << height << ' ' << SDL_GetError () << endl;
		exit(1);
	}

  if (SDL_GL_SetAttribute (SDL_GL_DOUBLEBUFFER, 1) < 0) {
		cout << "SDL/GL double buffer init failed: " << SDL_GetError () << endl;
		exit(1);
	}

	extern string APP_NAME;
	const char* name = APP_NAME.c_str ();
	SDL_WM_SetCaption (name, name);

  cx = cy = 0;
  width=height=0;
  xmax = ymax = 0;
  xmax_1 = ymax_1 = 0;

}

bool viewport::inside (int x, int y) {
	bool xi = (x >= 0) && (x <= xmax);
	bool yi = (y >= 0) && (y <= ymax);
	return (xi && yi);
}

int window::PAN_RATE = 100, window::ZOOM_RATE = 100;
double window::PAN_REPEAT = 1. / PAN_RATE, window::ZOOM_REPEAT = 1. / ZOOM_RATE;
float window::PAN_AMOUNT = 0.1, window::ZOOM_AMOUNT = 0.1;

void window::calc_panxy () {
  pan_x = PAN_AMOUNT * width;
  pan_y = PAN_AMOUNT * height;
}

void window::calc_handle_radius () {
	handle_radius = viewport::HANDLE_FACTOR * width;
	handle_radius2 = handle_radius * handle_radius;
}

window::window () : box<float>() {
  width_1 = height_1 = 0;
  aspect_ratio = 0;
}

window::window (float l, float b, float r, float t) {
  set (l, b, r, t);
}

void window::set (float l, float b, float r, float t) {
  operator() (l, b, r, t);
  width_1 = 1.0 / width;
  height_1 = 1.0 / height;
  aspect_ratio = width * 1.0 / height;
  calc_panxy ();
}

void window::get (float& l, float& b, float& r, float& t) {
  l = left;
  b = bottom;
  r = right;
  t = top;
}

void window::panx (int dir) {
  left = left + pan_x * dir;
  right = left + width;
}

void window::pany (int dir) {
  bottom = bottom + pan_y * dir;
  top = bottom + height;
}

void window::zoom (int dir) {

  float dw = ZOOM_AMOUNT * width * dir, dh = dw / aspect_ratio, dw2 = dw / 2, dh2 = dh / 2;
  float l = left - dw2;
  float r = right + dw2;
  float b = bottom - dh2;
  float t = top + dh2;
  float w = r - l, h = t - b;
  if ((w > 0) && (h > 0)) {
		set (l, b, r, t);
		calc_handle_radius ();
	}
}

void window::calc_repeats () {
	if (PAN_RATE > 0) PAN_REPEAT = 1. / PAN_RATE;
	if (ZOOM_RATE > 0) ZOOM_REPEAT = 1. / ZOOM_RATE;
}